'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEFAULT_OPTIONS = exports.ABSINTHE_OPTIONS = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.createNetworkInterface = createNetworkInterface;

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _networkInterface = require('apollo-client/transport/networkInterface');

var _phoenix = require('phoenix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ABSINTHE_OPTIONS = exports.ABSINTHE_OPTIONS = {
  channel: {
    topic: '__absinthe__:control',
    event: 'doc'
  },
  subscription: function subscription(subResponse) {
    return {
      topic: subResponse.subscriptionId,
      event: 'subscription:data',
      map: function map(payload) {
        return payload.result.data;
      },
      off: function off(controlChannel) {
        controlChannel.push('unsubscribe', {
          subscriptionId: subResponse.subscriptionId
        });
      }
    };
  }
};

var DEFAULT_OPTIONS = exports.DEFAULT_OPTIONS = ABSINTHE_OPTIONS;

var applyWares = function applyWares(ctx, wares) {
  return new Promise(function (resolve, reject) {
    var queue = function queue(funcs, scope) {
      var next = function next() {
        if (funcs.length > 0) {
          var f = funcs.shift();
          f && (f.applyMiddleware || f.applyAfterware).apply(scope, [ctx, next]);
        } else {
          resolve(ctx);
        }
      };
      next();
    };
    queue(wares.slice());
  });
};

var createSocket = function createSocket(options) {
  var uri = options.uri;

  var Socket = options.Socket || function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return new (Function.prototype.bind.apply(_phoenix.Socket, [null].concat(args)))();
  };
  return Socket(uri, options);
};

var getSocket = function getSocket(sockets, options) {
  if (options.logger === true) {
    options.logger = function (topic, msg, data) {
      return console.log('Apollo websocket ' + topic + ': ' + msg, data);
    };
  }

  var uri = options.uri;

  if (!uri) throw 'Missing apollo websocket options.uri';

  return sockets[uri] || (sockets[uri] = {
    conn: createSocket(options),
    channels: {}
  });
};

var getChannel = function getChannel(socket, channelOptions) {
  if (!channelOptions || !channelOptions.topic) throw "Missing topic option";
  var topic = channelOptions.topic,
      params = channelOptions.params;

  return socket.channels[topic] || (socket.channels[topic] = socket.conn.channel(topic, params || {}));
};

var socketConnect = function socketConnect(sockets, options) {
  return new Promise(function (resolve, reject) {
    var socket = getSocket(sockets, options);
    if (socket.conn.isConnected()) {
      resolve(socket);
    } else {
      socket.conn.onOpen(function (_) {
        return resolve(socket);
      });
      socket.conn.onError(reject);
      socket.conn.connect();
    }
  });
};

var channelJoin = function channelJoin(socket, channelOptions) {
  return new Promise(function (resolve, reject) {
    var channel = getChannel(socket, channelOptions);
    if (channel.joinedOnce) {
      resolve(channel);
    } else if (!channel.isJoining()) {
      channel.join().receive('ok', function (_) {
        return resolve(channel);
      }).receive('error', reject).receive('timeout', reject);
    }
  });
};

var docEvent = function docEvent(_ref) {
  var channel = _ref.channel;
  return channel.event || channel.in_msg || 'doc';
};

var alwaysResolve = function alwaysResolve(promiser) {
  return function () {
    return promiser.apply(undefined, arguments).catch(function (err) {
      return err;
    });
  };
};

var performQuery = function performQuery(_ref2) {
  var channel = _ref2.channel,
      context = _ref2.context;
  return new Promise(function (resolve, reject) {
    var options = context.options,
        request = context.request;


    var message = docEvent(options);
    var payload = (0, _networkInterface.printRequest)(request);

    channel.push(message, payload).receive('ok', resolve).receive('ignore', resolve).receive('error', reject).receive('timeout', reject);
  });
};

var NO_SUBSCRIPTION_OPTIONS = '\nexpected options.subscription to be a function with signature:\n\n   subscriptionResponse => ({topic, event})\n\nthat is, given the server\'s subscription response, it must return\nan object with two things, the topic and the event name\nwhere to expect incoming data for the subscription.\n\nFor example, if you are using Absinthe backend, this function can be like:\n\n\n      subscription: subResponse => ({\n        topic: subResponse.subscriptionId,\n        event: \'subscription:data\',\n        map: payload => payload.result.data,\n        off: controlChannel => {\n          controlChannel.push(\'unsubscribe\', {\n            subscriptionId: subResponse.subscriptionId\n          })\n        }\n      })\n\n';

var subscriptionOptions = function subscriptionOptions(options) {
  return function (subscriptionResponse) {
    return new Promise(function (resolve, reject) {
      var subscription = options.subscription;

      var subOptions = typeof subscription === 'function' ? subscription(subscriptionResponse) : {};
      if ((typeof subOptions === 'undefined' ? 'undefined' : _typeof(subOptions)) !== 'object' || !subOptions.topic || !subOptions.event) {
        throw NO_SUBSCRIPTION_OPTIONS;
      }
      resolve(subOptions);
    });
  };
};

var subscribeToFastlane = function subscribeToFastlane(_ref3) {
  var channel = _ref3.channel,
      subCallback = _ref3.subCallback,
      responseMiddleware = _ref3.responseMiddleware;
  return function (subOptions) {
    return new Promise(function (resolve, reject) {

      var handleData = _ramda2.default.pipeP(function (payload) {
        return Promise.resolve(payload);
      }, subOptions.map || _ramda2.default.identity, responseMiddleware, function (responseContext) {
        var response = responseContext.response;

        subCallback( /*error*/null, response);
        return responseContext;
      });

      var sub = {
        handler: function handler(incoming) {
          return incoming.topic === subOptions.topic && incoming.event === subOptions.event && handleData(incoming.payload);
        }

        // listen on the socket fastlane
      };channel.socket.onMessage(function (incoming) {
        sub.handler && sub.handler(incoming);
        return incoming;
      });

      var unsubscribe = function unsubscribe(_) {
        (subOptions.off || _ramda2.default.identity)(channel);
        delete sub.handler;
      };

      resolve({ unsubscribe: unsubscribe });
    });
  };
};

var performSubscribe = function performSubscribe(_ref4) {
  var subCallback = _ref4.subCallback,
      responseMiddleware = _ref4.responseMiddleware;
  return function (_ref5) {
    var channel = _ref5.channel,
        context = _ref5.context;

    return _ramda2.default.pipeP(performQuery, subscriptionOptions(context.options), subscribeToFastlane({ channel: channel, subCallback: subCallback, responseMiddleware: responseMiddleware }))({ channel: channel, context: context });
  };
};

var senderChannel = function senderChannel(sockets) {
  return function (context) {
    var options = context.options;

    return _ramda2.default.pipeP(function (_) {
      return socketConnect(sockets, options);
    }, function (socket) {
      return channelJoin(socket, options.channel);
    }, function (channel) {
      return { channel: channel, context: context };
    })();
  };
};

var isEmptyOrNil = _ramda2.default.either(_ramda2.default.isEmpty, _ramda2.default.isNil);

var responseData = function responseData(_ref6) {
  var response = _ref6.response;

  return new Promise(function (resolve, reject) {
    if (isEmptyOrNil(response)) {
      reject('No response');
    } else if (!isEmptyOrNil(response.data)) {
      resolve(response);
    } else {
      reject(response);
    }
  });
};

function createNetworkInterface(opts) {
  var ifaceOpts = _ramda2.default.mergeDeepRight(DEFAULT_OPTIONS, opts);

  var sockets = {};
  var middlewares = [];
  var afterwares = [];

  var use = _ramda2.default.map(function (item) {
    return middlewares.push(item);
  });
  var useAfter = _ramda2.default.map(function (item) {
    return afterwares.push(item);
  });

  var requestMiddleware = function requestMiddleware(request) {
    var options = _ramda2.default.clone(ifaceOpts);
    return applyWares({ request: request, options: options }, middlewares);
  };

  var responseMiddleware = function responseMiddleware(response) {
    var options = _ramda2.default.clone(ifaceOpts);
    return applyWares({ response: response, options: options }, afterwares);
  };

  var query = _ramda2.default.pipeP(requestMiddleware, senderChannel(sockets), alwaysResolve(performQuery), responseMiddleware, responseData);

  var subscribe = function subscribe(request, subCallback) {
    return _ramda2.default.pipeP(requestMiddleware, senderChannel(sockets), performSubscribe({ subCallback: subCallback, responseMiddleware: responseMiddleware }))(request);
  };

  var unsubscribe = function unsubscribe(subscription) {
    subscription.then(function (_ref7) {
      var unsubscribe = _ref7.unsubscribe;
      return unsubscribe();
    });
  };

  return { query: query, use: use, useAfter: useAfter, subscribe: subscribe, unsubscribe: unsubscribe };
}